﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using UnityEngine;
using UnityEngine.Events;
using UnityEngine.UI;


public class GadgetManager : MonoBehaviour, ISerializationCallbackReceiver
{
    // could be worked around of. But as long as not needed: this is better
    public static GadgetManager Singelton = null;

    /// <summary>Which cursor to use</summary>
    /// <remarks>When not set in Inspector, will be searching for any <see cref="WorldCursor"/>.</remarks>
    public static WorldCursor Cursor;
    public static LineRenderer LineRenderer;
    public static GameObject GadgetButton;
    public static GameObject ParentMe;
    public static Material[] Materials;
    public static Sprite[] ButtonSprites;

    public static Dictionary<Gadget.GadgetIDs, DataContainerGadgetInit> DataContainerGadgetDict;

    public static float activeGadgetScaleFactor = 1.5f;

    public static int activeGadget;
    public static Gadget[] gadgets;
    public static Button[] buttons;

    private static UnityAction<RaycastHit> OnHit;
    private static RectTransform updateRect;

    #region Unity Serialization
    [SerializeField] private GameObject _GadgetButton;
    [SerializeField] private GameObject _ParentMe;
    [SerializeField] private WorldCursor _Cursor;
    [SerializeField] private LineRenderer _LineRenderer;

    [SerializeField] private DataContainerGadgetCollection _DataContainerGadgetDict;

    public void OnBeforeSerialize() { }

    public void OnAfterDeserialize()
    {
        Cursor = _Cursor;
        LineRenderer = _LineRenderer;
        GadgetButton = _GadgetButton;
        ParentMe = _ParentMe;

        Materials = _DataContainerGadgetDict?.Materials ?? new Material[0];
        ButtonSprites = _DataContainerGadgetDict?.ButtonSprites ?? new Sprite[0];
        DataContainerGadgetDict = _DataContainerGadgetDict?.DataContainerGadgetDict ?? new();
    }
    #endregion

    private void OnDestroy()
    {
        Singelton = null;
    }

    private void Awake()
    {
        Singelton ??= this;
        if (this != Singelton) { 
            Debug.LogError("Only one Instance of GadgetManager allowed!");
            return;
        }

        gameObject.DestroyAllChildren();

        if (Cursor == null)
            Cursor = GameObject.FindObjectOfType<WorldCursor>();
        if (LineRenderer == null)
            LineRenderer = Cursor.GetComponent<LineRenderer>();
        if (ParentMe == null)
            ParentMe = this.gameObject;

        updateRect = this.gameObject.transform.parent as RectTransform;

        if (UIconfig.FrameITUIversion == 1)
            activeGadgetScaleFactor = 1.5f;
        else if (UIconfig.FrameITUIversion == 2)
            activeGadgetScaleFactor = 2.1f;


        gadgets = Gadget.GadgetTypes.Select(t => (Gadget) Activator.CreateInstance(t)).OrderBy(g => g.Rank).ToArray();
        buttons = new Button[gadgets.Length];

        for (int i = 0; i < gadgets.Length; i++) {
            gadgets[i].Awake();
            //gadgets[i].Start();
            CreateButton(i);
        }
    }

    void Start()
    {
        activeGadget = 0;
        buttons[0].transform.localScale *= activeGadgetScaleFactor;
        OnHit = gadgets[0].Hit;
        ActivateGadget(0);
    }

    void Update()
    {
        if (this != Singelton) return;

        if (!ParentMe.activeInHierarchy)
            return;

        int offset = 0;

        if (Input.GetButtonDown("ToolMode"))
            offset = 1;
        else if (Input.GetAxis("Mouse ScrollWheel") != 0)
            offset = (int) Mathf.Sign(Input.GetAxis("Mouse ScrollWheel"));

        if (offset != 0)
            ActivateGadget(activeGadget + offset);

        gadgets[activeGadget].Update();
    }

    void OnDisable()
    {
        if (this != Singelton) return;

        gadgets[activeGadget].Disable();
    }

    void OnEnable()
    {
        if (this != Singelton) return;

        gadgets[activeGadget].Enable();
    }

    private void CreateButton(int gid)
    {
        GameObject button = GameObject.Instantiate(GadgetButton);
        button.transform.SetParent(ParentMe.transform);

        button.GetComponent<Image>().sprite = ButtonSprites[gadgets[gid].ButtonIndx];
        var cache = gameObject.transform.parent.parent as RectTransform;
        (buttons[gid] = button.GetComponent<Button>())
            .onClick.AddListener(() => ActivateGadget(gid));
    }

    public static void ActivateGadget(int gid)
    {
        ParentMe.SetActive(true);
        gid += gid >= gadgets.Length 
            ? -gadgets.Length 
            : gid < 0 ? +gadgets.Length : 0;

        //buttons[activeGadget].animator.StopPlayback();
        buttons[activeGadget].transform.localScale /= activeGadgetScaleFactor;
        gadgets[activeGadget].Disable();

        activeGadget = gid;
        //buttons[gid].animator.StartPlayback();
        buttons[gid].transform.localScale *= activeGadgetScaleFactor;
        gadgets[gid].Enable();

        CommunicationEvents.TriggerEvent.RemoveListener(OnHit);
        CommunicationEvents.TriggerEvent.AddListener(OnHit = gadgets[gid].Hit);

        CommunicationEvents.ToolModeChangedEvent.Invoke(gid);
        LayoutRebuilder.ForceRebuildLayoutImmediate(updateRect);
    }
}
