﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using static CommunicationEvents;

public class LineTool : Gadget
{
    //Variables for LineMode distinction
    private bool LineModeIsFirstPointSelected = false;
    private Fact LineModeFirstPointSelected = null;


    public override void _Hit(RaycastHit hit)
    {
        if (hit.transform.gameObject.layer != LayerMask.NameToLayer("Point"))
            return;

        Fact tempFact = StageStatic.stage.factState[hit.transform.GetComponent<FactObject>().URI];

        //If first point was already selected AND second point != first point
        if (LineModeIsFirstPointSelected && LineModeFirstPointSelected.Id != tempFact.Id)
        {
            //Create LineFact
            FactManager.AddRayFact(LineModeFirstPointSelected.Id, tempFact.Id);
            ResetGadget();
        }
        else
        {
            //Activate LineDrawing for preview
            LineModeIsFirstPointSelected = true;
            LineModeFirstPointSelected = tempFact;
            ActivateLineDrawing();
        }
    }

    protected override void _ResetGadget()
    {
        LineModeIsFirstPointSelected = false;
        LineModeFirstPointSelected = null;
    }

    protected override void _ActivateLineDrawing()
    {
        GadgetManager.LineRenderer.enabled = true;
        GadgetManager.LineRenderer.positionCount = 2;

        GadgetManager.LineRenderer.startWidth = 0.095f;
        GadgetManager.LineRenderer.endWidth = 0.095f;
        //Add the position of the Fact for the start of the Line
        SetPosition(0, LineModeFirstPointSelected.Representation.transform.position);
        //The second point is the same point at the moment
        SetPosition(1, LineModeFirstPointSelected.Representation.transform.position);
    }

    //Updates the second-point of the Line when First Point was selected in LineMode
    protected override void _UpdateLineDrawing()
    {
        SetPosition(1, GadgetManager.Cursor.transform.position);
    }
}
