﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using static CommunicationEvents;

public class Tape : Gadget
{
    //Variables for TapeMode distinction
    private bool TapeModeIsFirstPointSelected = false;
    private Fact TapeModeFirstPointSelected = null;


    public override void _Hit(RaycastHit hit)
    {
        if (hit.transform.gameObject.layer != LayerMask.NameToLayer("Point"))
            return;

        Fact tempFact = StageStatic.stage.factState[hit.transform.GetComponent<FactObject>().URI];

        //If first point was already selected AND second point != first point
        if (TapeModeIsFirstPointSelected && TapeModeFirstPointSelected.Id != tempFact.Id)
        {
            //Create LineFact
            FactManager.AddLineFact(TapeModeFirstPointSelected.Id, tempFact.Id);
            ResetGadget();
        }
        else
        {
            //Activate LineDrawing for preview
            TapeModeIsFirstPointSelected = true;
            TapeModeFirstPointSelected = tempFact;
            ActivateLineDrawing();
        }
    }

    protected override void _ResetGadget()
    {
        TapeModeIsFirstPointSelected = false;
        TapeModeFirstPointSelected = null;
    }

    protected override void _ActivateLineDrawing()
    {
        GadgetManager.LineRenderer.positionCount = 2;

        GadgetManager.LineRenderer.startWidth = 0.095f;
        GadgetManager.LineRenderer.endWidth = 0.095f;
        //Add the position of the Fact for the start of the Line
        SetPosition(0, TapeModeFirstPointSelected.Representation.transform.position);
        //The second point is the same point at the moment
        SetPosition(1, TapeModeFirstPointSelected.Representation.transform.position);
    }

    //Updates the second-point of the Line when First Point was selected in LineMode
    protected override void _UpdateLineDrawing()
    {
        SetPosition(1, GadgetManager.Cursor.transform.position);
    }
}